/*
 * Copyright (c) 2013 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 * 
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */

#include <stdio.h>
#include <asl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/dir.h>
#include <string.h>
#include <sys/sysctl.h>

/* Log files generated by RootDomain */
#define RD_STACKS "/var/tmp/SleepWakeStacks.dump"
#define RD_LOG    "/var/tmp/SleepWakeLog.dump"
#define RDTO_STACKS "/var/tmp/SleepWakeTimeoutStacks.dump"
#define RDTO_LOG    "/var/tmp/SleepWakeTimeoutLog.dump"

/* Log files generated by powerd */
#define POWERD_STACKS   "/var/tmp/PMSleepWakeStacks.dump"
#define POWERD_LOG      "/var/tmp/PMSleepWakeLog.dump"

#define FILENAMETOKEN "Sleep Wake Failure"

#define PANIC_FILE "/var/db/PanicReporter/current.panic"

void process_logfiles(char *stacksfile, char *logfile)
{
    struct stat stacks;
    struct stat logs;
    char cmd[200];
    DIR *dirp;
    struct dirent *dp;
    char fname[100], swfile[100];
    long crtime = 0;
    struct stat fstats, pfstat;
    struct timeval sleeptime, boottime;
    size_t size;


    if ((lstat(logfile, &logs) != 0) || !S_ISREG(logs.st_mode))
        return;

    if (lstat(stacksfile, &stacks)) {
        // For cases where there is no stack file generated
        snprintf(cmd, sizeof(cmd), "/usr/bin/touch %s", stacksfile);
        system(cmd);
    }
    else if (!S_ISREG(S_IFREG)) {
        // If file exists, it should be a regular file with no links
        return;
    }

    snprintf(cmd, sizeof(cmd),
            "/usr/sbin/spindump -sleepwakefailure_stackshot_file %s -sleepwakefailure_data_file %s",
            stacksfile, logfile);
    system(cmd);


    unlink(stacksfile);
    unlink(logfile);

    if (strncmp(logfile, RD_LOG, sizeof(RD_LOG)) != 0)
        return;

    // Due diligence check to make sure this file is not generated due to system sleep
    size = sizeof(sleeptime);
    if ((sysctlbyname("kern.sleeptime", &sleeptime, &size, NULL, 0) != 0) || (sleeptime.tv_sec != 0))
        return;

    // For RD_LOG files, trigger a pop-up dialog to report that
    // system has rebooted due to Sleep Wake failure.
    //
    // Look for most recently created Sleep Wake Failure file
    dirp = opendir("/Library/Logs/DiagnosticReports");
    if (dirp == NULL) {
        return ;
    }

    while ((dp = readdir(dirp)) != NULL) {
        if ((strnstr(dp->d_name, FILENAMETOKEN, sizeof(FILENAMETOKEN)) == dp->d_name) 
                    && (dp->d_type == DT_REG)) {
            snprintf(fname, sizeof(fname), "/Library/Logs/DiagnosticReports/%s", dp->d_name);
            stat(fname, &fstats);

            if (S_ISREG(fstats.st_mode) && (crtime < fstats.st_birthtime)) {
                crtime = fstats.st_birthtime;
                strncpy(swfile, fname, sizeof(swfile));
            }
        }
    }

    // Check for existence of SleepWakeFailure log file
    if (crtime == 0)
        return;     

    // Due diligence check to make sure this file is generated after system boot
    size = sizeof(boottime);
    if ((sysctlbyname("kern.boottime", &boottime, &size, NULL, 0) != 0) || (boottime.tv_sec > crtime))
        return;

    // If panic file link already exists, skip linking this Sleep Wake failure file
    if (lstat(PANIC_FILE, &pfstat) == 0)
        return;

    symlink(swfile, PANIC_FILE);

}

int main(int argc, char **argv)
{

    process_logfiles(POWERD_STACKS, POWERD_LOG);
    process_logfiles(RD_STACKS, RD_LOG);
    process_logfiles(RDTO_STACKS, RDTO_LOG);


    return 0;
}
